#ifndef _DAP_GUC_OPTIMIZATION_H
#define _DAP_GUC_OPTIMIZATION_H

#include <stdint.h>
#include "sqlite3.h"

#define MAX_SAMPLES 16

// Struct which contains information about a transfer, including how much of
// the file has been transferred and information used by the optimization
// service.
struct transfer_status {
	// Information about the transfer as a whole
	char *src_url;
	char *dest_url;
	uintmax_t size;  // Filesize in bytes
	uintmax_t off;   // How many bytes have been transferred
	uint8_t streams; // The number of parallel streams to use

	// Information used by the optimization service
	uintmax_t chunk; // Size of a "chunk" in bytes
	uintmax_t len;   // Bytes to be transferred before re-running the service
	uint32_t sample; // The sampling size

	// Bit flags
	int optimized:1; // Whether or not to do optimization.
	int dynamic:1;   // Whether or not to do dynamic optimization.
	int history:1;   // Whether or not to use the history database.
};

// Struct containing the results of a single sampling.
struct thrp_sample_res {
	uint8_t sc;   // Stream count
	double thrp;  // Associated throughput
};

// Struct representing a container for sample results.
struct thrp_samples {
	unsigned int count;
	struct thrp_sample_res res[MAX_SAMPLES];
};

// Add a sample to a sample collection. Return 0 if the sample has more
// samples than MAX_SAMPLES allows. Otherwise return the number of samples.
int add_sample(uint8_t streams, double throughput, thrp_samples *samples);

int do_optimization_service(transfer_status *ts);

int do_analysis(thrp_samples *tps, double *peak_thrp, double *single_thrp);

void do_sampling(transfer_status *ts, thrp_samples *tps);

double get_throughput(transfer_status *ts);
char *est_tran_time(uintmax_t filesize, double throughput);
inline void print_est_tran_time(uintmax_t filesize, double throughput);

inline double cal_err(thrp_sample_res *tsr, int tsr_c,
                      double a, double b, double c, int *df);

inline double cal_dif(thrp_sample_res tsr1, thrp_sample_res tsr2);

inline double cal_a(thrp_sample_res *tsr, int i, int j, int k);

inline double cal_b(thrp_sample_res *tsr, int i, int j, int k);

inline double cal_c(thrp_sample_res *tsr, int i, int j, int k);

uint8_t cal_full_peak(double a, double c);

uintmax_t filesize_in_bytes(char *fs);

void write_classad_optimization (int opt1, double thrp, double difftime, char *est_time, uintmax_t filesize, char *est_time1);

#endif
