/***************************************************************
*
* Copyright (C) 2008-2010, Distributed Systems Laboratory,
* Center for Computation & Technology, and Department of Computer Science
* Louisiana State University, Baton Rouge, LA.
*
* Licensed under the Apache License, Version 2.0 (the "License"); you
* may not use this file except in compliance with the License.  You may
* obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
***************************************************************/

#ifndef _DAP_LOGGER_H_
#define _DAP_LOGGER_H_

#include "dap_stream_list.h"
#include "condor_common.h"
#include "condor_string.h"
#include "condor_debug.h"
#include "condor_event.h"
#include "condor_daemon_core.h"

#ifndef WANT_CLASSAD_NAMESPACE
#define WANT_CLASSAD_NAMESPACE
#endif
#include "condor_fix_iostream.h"
#include "classad/classad_distribution.h"

// Userlog events defined per Condor standards
#define ULOG_SUBMIT  0
#define ULOG_EXECUTE 1
#define ULOG_TERM    5       
#define ULOG_ABORT   9
#define ULOG_GENERIC 8

// Alt. names for ULOG_TERM and ULOG_ABORT for compatibility with Condor.
#define ULOG_JOB_TERMINATED 5
#define ULOG_JOB_ABORTED    9

struct userlog_t {
    int dap_id;
	userlog_t *next, *prev;

	char *owner;
	stream_list *out;
	int proc;
	bool log_xml;
};

extern userlog_t *head_ulog, *tail_ulog;

void write_dap_log(const char *logfilename, const char *status,
                   const char *param1, const char *value1,
				   const char *param2 = NULL, const char *value2 = NULL,
				   const char *param3 = NULL, const char *value3 = NULL,
				   const char *param4 = NULL, const char *value4 = NULL,
				   const char *param5 = NULL, const char *value5 = NULL,
				   const char *param6 = NULL, const char *value6 = NULL);

void write_classad_log(const char *logfilename, const char *status,
                       const classad::ClassAd *classad);

void write_collection_log(classad::ClassAdCollection *dapcollection,
                          const char *dap_id, const char *update);

bool ulog_submit(char **b_str, const classad::ClassAd *ad);
bool ulog_execute(char **b_str, const classad::ClassAd *ad);
bool ulog_term(char **b_str, const classad::ClassAd *ad);
bool ulog_abort(char **b_str, const classad::ClassAd*);
bool ulog_generic(char **b_str, const classad::ClassAd *ad);

bool write_ulog_event(userlog_t *ulog, int event, const classad::ClassAd *ad);

userlog_t *new_ulog(const classad::ClassAd *ad);
void close_ulog(userlog_t *ulog);

void ulog_list_add(userlog_t *ul);
void ulog_list_del(userlog_t *ul);
void ulog_list_del(int dap_id);
void ulog_unlink(userlog_t *ul);
userlog_t *ulog_list_find(int dap_id);

bool ulog_register(const classad::ClassAd *ad, void *obj, int type);
bool ulog_register(const classad::ClassAd *ad, FILE *fp);
bool ulog_register(const classad::ClassAd *ad, ReliSock *rs);
bool ulog_register(const classad::ClassAd *ad, const char *path);
bool ulog_register(const classad::ClassAd *ad);

void ulog_unregister(int dap_id);
void ulog_unregister(const classad::ClassAd *ad);

bool userlog(const classad::ClassAd *ad, int event);

#endif
