/***************************************************************
*
* Copyright (C) 2008-2010, Distributed Systems Laboratory,
* Center for Computation & Technology, and Department of Computer Science
* Louisiana State University, Baton Rouge, LA.
*
* Licensed under the Apache License, Version 2.0 (the "License"); you
* may not use this file except in compliance with the License.  You may
* obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
***************************************************************/

/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 *
 * Condor Software Copyright Notice
 * Copyright (C) 1990-2006, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 *
 * This source code is covered by the Condor Public License, which can
 * be found in the accompanying LICENSE.TXT file, or online at
 * www.condorproject.org.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * AND THE UNIVERSITY OF WISCONSIN-MADISON "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, OF SATISFACTORY QUALITY, AND FITNESS
 * FOR A PARTICULAR PURPOSE OR USE ARE DISCLAIMED. THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS AND THE UNIVERSITY OF WISCONSIN-MADISON
 * MAKE NO MAKE NO REPRESENTATION THAT THE SOFTWARE, MODIFICATIONS,
 * ENHANCEMENTS OR DERIVATIVE WORKS THEREOF, WILL NOT INFRINGE ANY
 * PATENT, COPYRIGHT, TRADEMARK, TRADE SECRET OR OTHER PROPRIETARY
 * RIGHT.
 *
 ****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "dap_srb.h"
#include "dap_constants.h"
#include "dap_error.h"
#include "dap_url.h"
#include "dap_srb_util.h"
// Stork SRB transfers files > 2GB
#ifndef _LARGEFILE_SOURCE
#error _LARGEFILE_SOURCE not defined!
#endif
#ifndef _LARGEFILE64_SOURCE
#error _LARGEFILE64_SOURCE not defined!
#endif
#if _FILE_OFFSET_BITS != 64
#error _FILE_OFFSET_BITS not 64
#endif
#define DATATYPE        "ascii text"		/* FIXME: hard-coded */

/*
 * Things that need fixing:
 *    DATATYPE
 *    catType/CAT_TYPE/MDAS_CATALOG (hard coded)
 *    srbAuthScheme (hard coded)
 *
 */

#define BUFSIZE 1024*1024
#define MAX_STR 512

char *srbAuthScheme = "ENCRYPT1";
char *serverDn = NULL;
int catType = MDAS_CATALOG;

//---
//definition for restartState
#define PATH_MATCHING           0x1
#define LAST_PATH_MATCHED       0x2
#define MATCHED_RESTART_COLL    0x4
#define OPR_RESUMED             0x8
#define MAX_NAME_LEN   1088 // 1024 + 64


typedef struct {
    char restartFile[MAX_NAME_LEN];
    int fd;             /* df of the opened restartFile */
    int doneCnt;        /* a count of number of files done in the coll/dir */
    char collection[MAX_NAME_LEN];      /* the coll/dir being restarted */
    char lastDonePath[MAX_NAME_LEN];    /* the last path that is done */
    int curCnt;
    int restartState;
} srbRestart_t;


typedef struct srbPath {
    //objType_t objType;
    //objStat_t objState;
    //rodsLong_t size;
    uint objMode;
    char inPath[MAX_NAME_LEN];   /* input from commnad line */
    char outPath[MAX_NAME_LEN];  /* the path after parsing the inPath */
    //char dataId[NAME_LEN];
    //char chksum[NAME_LEN];
    //rodsObjStat_t *rodsObjStat;
} srbPath_t;

typedef enum {  /* object type */
    //UNKNOWN_OBJ_T,
    DATA_OBJ_T,
    COLL_OBJ_T,
    //UNKNOWN_FILE_T,
    LOCAL_FILE_T,
    //LOCAL_DIR_T,
    //NO_INPUT_T,
} objType_t;


char STORK_DAPID[MAX_STR] = "";
srbRestart_t srbRestart;

//---
char file_ckpt_str[] = "stork_file.rescue.";
//===

//===
/* ============================================================================
 * Create a connection to SRB
 * ==========================================================================*/

srbConn* connect_to_srb(Url& srbParsed){

  srbConn *conn;

  //Open a connection to SRB server
  conn = srbConnect (srbParsed.host(), srbParsed.port(),
      srbParsed.password(), srbParsed.srbUser(), srbParsed.srbMdasDomain(),
      srbAuthScheme, serverDn);

  //error unable to open connection to SRB server
  if (clStatus(conn) != CLI_CONNECTION_OK){

    fprintf(stderr,"srbConnect failed.\n");
    fprintf(stderr,"=================================\n");
    fprintf(stderr,"%s",clErrorMessage(conn));
    fprintf(stderr,"=================================\n");
    srb_perror (2, clStatus(conn), "", SRB_RCMD_ACTION|SRB_LONG_MSG);
    fprintf(stderr,"=================================\n");
    fprintf(stderr,"host = '%s'\n", srbParsed.host() );
    fprintf(stderr,"port = '%s'\n", srbParsed.port() );
    //fprintf(stderr,"auth = '%s'\n", srbParsed.passwd() );
    fprintf(stderr,"user = '%s'\n", srbParsed.srbUser() );
    fprintf(stderr,"domainName = '%s'\n", srbParsed.srbMdasDomain() );
    fprintf(stderr,"authScheme = '%s'\n", "ENCRYPT1");
    fprintf(stderr,"GSI server DN = '%p'\n", (void *)NULL);
    clFinish(conn);
    exit(-1);

  }

  return conn;
}


/* ============================================================================
 * Mkdir (or Collection) at SRB
 * ==========================================================================*/

int srb_mkdir(srbConn *conn, char *dst){

  int coll_fd, len = strlen(dst);
  char dirname[MAX_STR];
  char *basename;

  strncpy(dirname, dst, len);
  dirname[len] = '\0';

  basename = strrchr(dirname, '/');
  *basename  = '\0';
  basename++;

  coll_fd = srbCreateCollect(conn, catType, dirname, basename);
  if (coll_fd < 0 && coll_fd != -3220){
    // -3220 means directory already exists
    fprintf(stderr, "Error: can't create collection:.\n");
    return -1;
  }

  return 0;
}


/* ============================================================================
 * Mkdir at local filesystem
 * ==========================================================================*/

int file_mkdir(char *dst, int mymode){
  char *p = dst;

  do{
    p = strchr(p+1, '/');
    if(p) *p = '\0';
    
    // errno == 17 means that the directory already exists
    // we therefore ignore this error
    if(mkdir(dst, mymode) == -1 && errno != 17){
      fprintf(stderr, "mkdir %s failed: %s %d\n", dst, strerror(errno), errno);
      return -1;
    }
    if(p) *p = '/';
  }while(p);

  return 0;
}


//---

//checkpointing functions

int convertCharStr(char str1[], char *str2){

    //printf("strlen = %d\n", strlen(str1));
    int i = 0;
    while(*str2!= '\0' && i< MAX_NAME_LEN){
          str1[i] = *str2;
          //printf("%c\n", str1[i]);
          str2++;
          i++;
    }
}

int
getLineInBuf (char **inbuf, char *outbuf, int bufLen)
{
    char *inPtr, *outPtr;
    int bytesCopied  = 0;
    int c;

    inPtr = *inbuf;
    outPtr = outbuf;

    while ((c = *inPtr) != '\n' && c != EOF && bytesCopied < bufLen) {
        c = *inPtr;
        if (c == '\n' || c == EOF)
            break;
        *outPtr = c;
        inPtr++;
        outPtr++;
        bytesCopied++;
    }
    *outPtr = '\0';
    *inbuf = inPtr + 1;
    return (bytesCopied);
}

char *rstrcpy (char *dest, char *src, int maxLen)
{
    int len;

    if (dest == NULL || src == NULL)
        return NULL;

    len = strlen (src);

    if (len < 0 || len >= maxLen) {
        fprintf(stderr,
                "rstrcpy not enough space in dest, slen:%d, maxLen:%d",
                len, maxLen);
        return NULL;
    }

    if (strncpy (dest, src, len) != NULL) {
        dest[len] = '\0';
        return dest;
    } else {
        fprintf(stderr,
                "rstrcpy not enough space in dest, slen:%d, maxLen:%d",
                len, maxLen);
        return NULL;
    }
}


int
setStateForRestart (srbConn *conn, srbRestart_t *srbRestart, srbPath_t *targPath){

	if (srbRestart->restartState & PATH_MATCHING){
		//check the restart collection
		if (strstr(targPath->outPath, srbRestart->collection) != NULL){
			rstrcpy(targPath->outPath, srbRestart->collection, MAX_NAME_LEN);
			srbRestart->restartState |= MATCHED_RESTART_COLL;
			srbRestart->curCnt = 0;
		} else {
			//take out MATCHED_RESTART_COLL
			srbRestart->restartState = srbRestart->restartState &
							(~MATCHED_RESTART_COLL);
		}
	} else if (srbRestart->fd > 0){
		rstrcpy(srbRestart->collection, targPath->outPath, MAX_NAME_LEN);
		srbRestart->doneCnt = srbRestart->curCnt = 0;
	}
	return 0;
}


int
setStateForResume (srbConn *conn, srbRestart_t *srbRestart, char *restartPath, 
			objType_t objType, int deleteFlag){

//---
	char dirname[MAX_STR];
        char *basename = NULL; 
        int len = strlen(restartPath);
	struct stat64  destStat;
//===
	int status;

	if (restartPath != NULL && deleteFlag > 0) {
		if (objType == DATA_OBJ_T){

  			strncpy(dirname, restartPath, len); 
			//restartPath is not const, does strncpy work correctly?
  			dirname[len] = '\0';

  			basename = strrchr(dirname, '/');
  			*basename  = '\0';
  			basename++;

			if (!srbObjStat64(conn, catType, restartPath, &destStat)){
    				srbObjUnlink(conn, basename, dirname);
  			}
		
				
		} else if (objType == LOCAL_FILE_T) {
			status = unlink(restartPath);
		} else {
			fprintf(stderr, "setStateForResume:"
				"illegal objType %d for %s", 
				objType, restartPath);
		}
	}
	
	srbRestart->restartState = OPR_RESUMED; //resumed opr

	return 0;
}


// check the state for resume operation
//return 0 - skip
//return 1 - resume

int
chkStateForResume (srbConn *conn, srbRestart_t *srbRestart,
			char *targPath, objType_t objType, int deleteFlag){

	int status;
	
	//printf("curCnt=%d DoneCnt=%d targPath=%s\n", srbRestart->curCnt, srbRestart->doneCnt, targPath);

	if (srbRestart->restartState & MATCHED_RESTART_COLL){
	    if (srbRestart->curCnt > srbRestart->doneCnt){
		fprintf(stderr, "chkStateForResume: restart failed."
			"curCnt %d > doneCnt %d, path %s\n",
			srbRestart->curCnt, srbRestart->doneCnt, targPath);	
			return -1;
	    }

	    if (srbRestart->restartState & LAST_PATH_MATCHED){
		if (objType == DATA_OBJ_T || objType == LOCAL_FILE_T){
			// a file
			setStateForResume (conn, srbRestart, targPath,
					   objType, deleteFlag);
		}
		status = 1;
	    } else if (strcmp(targPath, srbRestart->lastDonePath) == 0){
		//will handle this with the next file
		srbRestart->curCnt ++;
		if (srbRestart->curCnt != srbRestart->doneCnt){
			fprintf(stderr,
				"chkStateForResume:Restart failed."
				"curCnt %d!=doneCnt %d, path %s",
				srbRestart->curCnt, srbRestart->doneCnt,
				targPath);
			return -1;
		}
		srbRestart->restartState |= LAST_PATH_MATCHED;	
		status = 0;
	    } else if (objType == DATA_OBJ_T || objType == LOCAL_FILE_T){
		//a file, no match, skip it
		srbRestart->curCnt ++;
		status = 0;
	    } else {
		//same directory, different path, drill down and see
		status = 1;
	    }
	} else if (srbRestart->restartState & PATH_MATCHING){
		//at the beginning state, so status = 0
		status = 0;
	} else {
		status = 1;
	}

	return status;

}



int
openRestartFile (char *restartFile, srbRestart_t *srbRestart){

	struct stat statbuf;
	char buf[MAX_NAME_LEN*3];
	char *inptr;
	char tmpStr[MAX_NAME_LEN];
	int status;
	
	status = stat (restartFile, &statbuf);

	if (status < 0 || statbuf.st_size == 0){
		srbRestart->fd = open (restartFile, O_RDWR|O_CREAT, 0644);
		if (srbRestart->fd < 0){
			//status = UNIX_FILE_OPEN_ERR - errno;
			status = -1;
			fprintf(stderr, "openRestartFile: open error for %s", restartFile);
			return status;
		}
		srbRestart->restartState = 0;
		printf("New restart %s opened\n", restartFile);
	} else if ((statbuf.st_mode & S_IFREG) == 0) {
		close (srbRestart->fd);
		srbRestart->fd = -1;
		//status = UNIX_FILE_OPEN_ERR;
		status = -1;
		fprintf(stderr, "openRestartFile: %s is not a file", restartFile);
		return status;
	} else {
		srbRestart->fd = open(restartFile, O_RDWR, 0644);
		if (srbRestart->fd < 0) {
			//status = UNIX_FILE_OPEN_ERR - errno;
			status = -1;
			fprintf(stderr, "openRestartFile: open error for %s", restartFile);
			return status;
		} 
		status = read (srbRestart->fd, (void *)buf, MAX_NAME_LEN * 3);
		if (status < 0)	{
			close (srbRestart->fd);
			//status = UNIX_FILE_READ_ERR - errno;
			status = -1;
			fprintf(stderr, "openRestartFIle: read error for %s", restartFile);
			return status;
		}

		inptr = buf;
		if(getLineInBuf(&inptr, srbRestart->collection, MAX_NAME_LEN) < 0) {
			fprintf(stderr, "openRestartFile: restartFile %s has 0 line",restartFile);
			//return USER_RESTART_FILE_INPUT_ERR;	
			return -1;
		}
		if (getLineInBuf (&inptr, tmpStr, MAX_NAME_LEN) < 0) {
			fprintf(stderr, "openRestartFile: restartFile %s has 1 line", restartFile);
			//return USER_RESTART_FILE_INPUT_ERR;	
			return -1;
		}
		srbRestart->doneCnt = atoi(tmpStr);

		if (getLineInBuf (&inptr, srbRestart->lastDonePath, MAX_NAME_LEN) < 0) {
			fprintf(stderr, "openRestartFile: restartFile %s has 2 line", restartFile);
			//return USER_RESTART_FILE_INPUT_ERR;	
			return -1;
		}

		srbRestart->restartState = PATH_MATCHING;
		printf("RestartFile %s opened\n", restartFile);
		printf("Restarting collection/directory = %s File count %d\n",
			srbRestart->collection, srbRestart->doneCnt);
		printf("File last completed = %s\n", srbRestart->lastDonePath);
	}
	return 0;
}

int
writeRestartFile (srbRestart_t *srbRestart, char *lastDonePath){

	char buf[MAX_NAME_LEN * 3];
	int status;
	
	srbRestart->doneCnt = srbRestart->curCnt;
	rstrcpy(srbRestart->lastDonePath, lastDonePath, MAX_NAME_LEN);
	memset(buf, 0, MAX_NAME_LEN * 3);
	snprintf(buf, MAX_NAME_LEN * 3, "%s\n%d\n%s\n",
		 srbRestart->collection, srbRestart->doneCnt, srbRestart->lastDonePath);
	
	lseek(srbRestart->fd, 0, SEEK_SET);
	status = write(srbRestart->fd, buf, MAX_NAME_LEN * 3);
	if (status != MAX_NAME_LEN * 3){
		fprintf(stderr, "WriteRestartFile: write error, errn = %d", errno);
		return -1;
	}
	return 0;
}


int initCondForChkPnt(srbRestart_t *srbRestart) {
	
	memset (srbRestart, 0, sizeof (srbRestart_t));
	int status;
	//---
	if (getenv("STORK_JOBID"))
		strcat(file_ckpt_str, getenv("STORK_JOBID"));
	//===
	status = openRestartFile(file_ckpt_str, srbRestart);
	if (status < 0){
		fprintf(stderr, "initCondForChkPnt: openRestartFile of"
			"%s error", file_ckpt_str);
		return status;
	}
	return 0;
}

//===

/* ============================================================================
 * Setting Stork specific parameters (filesize verify, chksum verify)
 * ==========================================================================*/
/*
int set_stork_parameters(int *filesizeVerify, int *checksumVerify, int *mymode, 
    int *nthreads, char *resource){
*/
//---
int set_stork_parameters(int *filesizeVerify, int *checksumVerify, int *checkPoint, 
			 int *mymode, int *nthreads, char *resource){	
//===
  char *envVal;

  /* Set file permissions as provided */
  if ((envVal=getenv("STORK_SET_PERMISSIONS")) != NULL){
    *mymode = atoi(envVal);
  }

  /* Perform checksum verification after data transfer */
  if ((envVal=getenv("STORK_VERIFY_FILESIZE"))&&(!strncasecmp("TRUE",envVal,4))){
    *filesizeVerify = 1;
  }

  /* Perform checksum verification after data transfer */
  if ((envVal=getenv("STORK_VERIFY_CHECKSUM"))&&(!strncasecmp("TRUE",envVal,4))){
    *checksumVerify = 1;
  }


//---
  if((envVal=getenv("STORK_CHECKPOINT_TRANSFER")) && (!strncasecmp("TRUE", envVal, 4))){
	*checkPoint = 1;	
	initCondForChkPnt(&srbRestart);
  }
//===

  /* Setting the number of threads for data transfer */
  if( *nthreads < 0){
    if((envVal=getenv("STORK_SET_PARALLELISM")) != NULL){
      //strcpy(STORK_SET_PERMISSIONS,envVal);
      *nthreads = atoi(envVal);
      if (*nthreads <= 0 || *nthreads == 1){
        *nthreads = 1;
      }
    }else{
      *nthreads = 4;
    }
  }else if(*nthreads <= 0 || *nthreads == 1){
    *nthreads = 1;
  }
  

  // Get the SRB resource type
  if ((envVal=getenv("STORK_SRB_RESOURCE")) != NULL){
    int len = strlen(envVal);
    strncpy(resource, envVal, len);
    resource[len] = '\0';
  }else{
    int len = strlen(srbResource());
    if(len > 0 && len < MAX_STR){
      strncpy(resource, srbResource(),len); 
      resource[len] = '\0';
    }
  }

  //fprintf(stdout, "f: %d c: %d m: %d n: %d\n", *filesizeVerify, *checksumVerify, *mymode, 
   //   *nthreads);

  return 0;

}


/* ============================================================================
 * Transfer a file from local disk to SRB
 * ==========================================================================*/

int transfer_from_file_to_srb(srbConn *conn, char *src, char *dst,
    int filesizeVerify, int checksumVerify, int mymode, int nthreads, char *resource){

  int out_fd, chksum_fd, stat_fd;
  FILE *p_fd;

  struct stat64 srcStat, destStat;
  char localchksum[10], srbchksum[10], chksumbuf[10 + strlen(src)];

  int chksum_flag = f_FLAG;
  char dirname[MAX_STR];
  char *basename = NULL;


  int len = strlen(dst);
  strncpy(dirname, dst, len);
  dirname[len] = '\0';

  basename = strrchr(dirname, '/');
  *basename  = '\0';
  basename++;

  // Check if file exists, if exists current policy is to 
  // overwrite file. We accomplish this by removing the file
  if (!srbObjStat64(conn, catType, dst, &destStat)){
    srbObjUnlink(conn, basename, dirname);
  }

  out_fd = srbObjPutC(conn, basename, dirname, resource, DATATYPE, NULL, src,
      -1,M_FLAG, nthreads);

  if (out_fd == SYS_ERR_NO_SUPPORT || out_fd == CLI_FUNC_NOT_SUPPORTED) {

    fprintf(stderr, "Error: No support for multiple threads. Transferring "\
        "data with a single thead %s %s %s\n", dirname, dst, src);

    out_fd = srbObjPut(conn, basename, dirname, resource, DATATYPE, NULL, src,
        -1,F_FLAG);
    
  } else if(out_fd < 0){
    fprintf(stderr, "Error: srbObjPutC unable to put the srb file: %s,"\
        "out_fd = %d.\n", dst, out_fd);
    fprintf(stderr,"%s",clErrorMessage(conn));
    srb_perror (2, clStatus(conn), "", SRB_RCMD_ACTION|SRB_LONG_MSG);
    srbObjUnlink(conn, basename, dirname);
    clFinish(conn);
    return -1;
  }

  //Retrieving the file size from the MCAT
  if(filesizeVerify){

    //Local file size
    if (lstat64(src, &srcStat) < 0){ //error unable to stat local file
      fprintf(stderr, "Error: Unable to stat source file: %s.\n", src);
      return -1;
    }

    stat_fd = srbObjStat64(conn, catType, dst, &destStat);
    if (stat_fd < 0){
      fprintf(stderr, "Error: unable to stat the srb file: %s, status = %d.\n",
          dst, stat_fd);
      return -1;
    }

    //Verifying source and destination file sizes
    if( ((srcStat.st_mode & S_IFREG) != 0) &&
        ((destStat.st_mode & S_IFREG) != 0)){
      if (srcStat.st_size != destStat.st_size){
        fprintf(stderr, "Error: filesizes differ src: %lu dest: %lu.\n",
            (unsigned long)srcStat.st_size, (unsigned long)destStat.st_size);
        clFinish(conn);
        return -1;
      }
    }
  }

  //Retrieving the checksum value from MCAT
  if(checksumVerify){
    chksum_fd = srbObjChksum(conn, basename, dirname,
        chksum_flag, 0, srbchksum);
    if (chksum_fd < 0){ //error unable to chksum srb file
      fprintf(stderr, "Error: unable to checksum srb file: %s.\n", dst);
      clFinish(conn);
      return -1;

    }

    //Calculating the checksum value for the source file
    sprintf (chksumbuf, "sum -s %s", src);

    p_fd = popen(chksumbuf, "r");
    if (p_fd == NULL){
      fprintf(stderr, "Error: unable to execute: %s.\n", chksumbuf);
      clFinish(conn);
      return -1;
    }

    fscanf(p_fd, "%s", localchksum);
    pclose(p_fd);

    //Verifying source and destination checksum values
    if (atoi(srbchksum) != atoi(localchksum)){
      fprintf(stderr, "Error: checksum mismatch srb: %d local: %d.\n",
          atoi(srbchksum), atoi(localchksum));
      fprintf(stderr, "Error: deleting the srb file.\n");
      srbObjUnlink(conn, basename, dirname);
      clFinish(conn);
      return -1;
    }
  }

  return 0;
}


/* ============================================================================
 * Transfer an SRB file to local disk
 * ==========================================================================*/

int transfer_from_srb_to_file(srbConn *conn, char *src, char *dst,
    int filesizeVerify, int checksumVerify, int mymode, int nthreads, char *resource){

  int out_fd = -1, stat_fd = -1, chksum_fd = -1;
  FILE *p_fd;
  struct stat64 srcStat, destStat;

  char *basename = NULL; 
  char dirname[MAX_STR];

  char localchksum[10], srbchksum[10], chksumbuf[10 + strlen(dst)];
  int chksum_flag = l_FLAG;
  //int bytesRead=0,bytesWritten=0;

  strcpy(dirname, src);
  basename = strrchr(dirname, '/');
  *basename  = '\0';
  basename++;

  //if destination file exists, unlink it.
  if (!lstat64(dst, &destStat)){
    if (unlink(dst)){
      fprintf(stderr, "unlink existing destination file %s: %s\n", 
          dst, strerror(errno));
      return -1;
    }
  }

  out_fd = srbObjGetC (conn, basename, dirname, dst, catType, nthreads);
  if (out_fd == SYS_ERR_NO_SUPPORT || out_fd == CLI_FUNC_NOT_SUPPORTED) {

    fprintf(stderr, "Error: No support for multiple threads. Transferring "\
        "data with a single thead %s %s %s\n", dirname, dst, src);

    out_fd = srbObjGet (conn, basename, dirname, dst);
    if(out_fd < 0){
      fprintf(stderr, "Error: Transfer %s %s failed.\n", src, dst);
      unlink(dst);
      return -1;
    }

  } else if(out_fd < 0){
    out_fd = srbObjGet (conn, basename, dirname, dst);
    if(out_fd < 0){
      fprintf(stderr, "Error: Transfer %s %s failed.\n", src, dst);
      unlink(dst);
      return -1;
    }

  }



  //Retrieving the file size from the MCAT
  if(filesizeVerify){

    //Local file size
    if (lstat64(dst, &destStat) < 0){ //error unable to stat local file
      fprintf(stderr, "Error: Unable to stat source file: %s.\n", src);
      return -1;
    }

    stat_fd = srbObjStat64(conn, catType, src, &srcStat);
    if (stat_fd < 0){
      fprintf(stderr, "Error: unable to stat the srb file: %s, status = %d.\n",
          src, stat_fd);
      return -1;
    }

    //Verifying source and destination file sizes
    if( ((srcStat.st_mode & S_IFREG) != 0) &&
        ((destStat.st_mode & S_IFREG) != 0)){
      if (srcStat.st_size != destStat.st_size){
        fprintf(stderr, "Error: filesizes differ src: %lu dest: %lu.\n",
            (unsigned long)srcStat.st_size, (unsigned long)destStat.st_size);
        clFinish(conn);
        return -1;
      }
    }
  }

  //Retrieving the checksum value from MCAT
  if(checksumVerify){
    chksum_fd = srbObjChksum(conn, basename, dirname, chksum_flag, 0,
        srbchksum);
    if (chksum_fd < 0){ //error unable to chksum srb file
      fprintf(stderr, "Error: unable to checksum srb file: %s.\n", src);
      clFinish(conn);
      return -1;
    }

    //Calculating the checksum value for the destination file
    sprintf (chksumbuf, "sum -s %s", dst);

    p_fd = popen(chksumbuf, "r");
    if (p_fd == NULL){
      fprintf(stderr, "Error: unable to execute: %s.\n", chksumbuf);
      clFinish(conn);
      return -1;
    }

    fscanf(p_fd, "%s", localchksum);
    pclose(p_fd);

    //Verifying source and destination checksum values
    if (atoi(srbchksum) != atoi(localchksum)){
      fprintf(stderr, "Error: checksum mismatch srb: %d local: %d.\n",
          atoi(srbchksum), atoi(localchksum));
      fprintf(stderr, "Error: deleting the srb file.\n");
      clFinish(conn);
      return -1;
    }
  }

  return 0;
}


/* ============================================================================
 * Performing transfers from local filesystem to SRB
 * Parse submitted URLs
 * Obtain Stork Parameters
 * Connect to SRB server
 * Perform transfers according to type
 *  - Recursive
 *  - wildcard
 *  - single file to data object
 *  - else print error message
 * ==========================================================================*/


int file_to_srb(char *src, char *dst, int nthreads){

  srbConn *conn;
  int status;

  int checksumVerify = 0, filesizeVerify = 0, mymode = 0700;
  char resource[MAX_STR];
  resource[MAX_STR-1] = '\0';

//---
   int checkPoint = 0;

   srbPath_t targPath;
   memset(&targPath, 0, sizeof(srbPath_t));
//===

  //------------------ Parsing URLs -----------------------------------------
  // Stork parse source url
  Url fileParsed(src);
  if (! fileParsed.parsed() ) {
    fprintf(stderr, "parse src URL '%s' error\n",
        fileParsed.urlNoPassword() );
    return -1;
  }

  // Stork parse destination url
  Url srbParsed(dst);
  if (! srbParsed.parsed() ) {
    fprintf(stderr, "parse dest URL '%s' error\n",
        srbParsed.urlNoPassword() );
    return -1;
  }

  // Stork parse the destination url as SRB
  srbParsed.parseSrb();
  if (! srbParsed.parsedSrb() ) {
    fprintf(stderr, "parse dest URL SRB fields'%s' error\n",
        srbParsed.urlNoPassword()
        );
    return -1;
  }

  if(fileParsed.path() == NULL || srbParsed.path() == NULL ||
      srbParsed.host() == NULL || srbParsed.port() == NULL ||
      srbParsed.srbUser() == NULL || srbParsed.srbMdasDomain() == NULL ||
      srbParsed.password() == NULL){
    fprintf(stderr, "Error: Some fields are missing. Please recheck your URL "\
      "and resubmit the job.\nsrc: %s\ndest: %s\n", fileParsed.urlNoPassword(),
      srbParsed.urlNoPassword());
    return -1;
  }
  //------------------ Finished Parsing URLs ---------------------------------



  //------------------ Obtain Stork Parameters -------------------------------

  //set_stork_parameters(&filesizeVerify, &checksumVerify, &mymode, &nthreads, resource);
  set_stork_parameters(&filesizeVerify, &checksumVerify, &checkPoint, &mymode, &nthreads, resource);

  //------------------ Finish obtaining Stork Parameters ---------------------



  //------------------ Connecting to SRB -------------------------------------

  conn = connect_to_srb(srbParsed);

  //------------------ Finished Connecting to SRB ----------------------------



  //------------------ Transfering files to irods ----------------------------
 
//---
	convertCharStr(targPath.outPath, srbParsed.dirname());
	setStateForRestart (conn, &srbRestart, &targPath);
//===

  //********************** Recursive Transfer ***************************/


  if(fileParsed.dirname() != NULL && srbParsed.dirname() != NULL &&
      fileParsed.basename() == NULL && srbParsed.basename() == NULL){

    // Perform recursive copy of the directory given in the source url

    FILE *infile;
    char path[MAX_STR];
    char dest[MAX_STR];
    char cmd[MAX_STR];

    // Create required directories at SRB
    sprintf(cmd, "find %s -type d", fileParsed.dirname());
    infile = popen(cmd,"r");
    if(infile == NULL){
      fprintf(stderr, "Error: Unable to open: tmp-SRBfilelist\n");
      return -1;
    }
    while(fgets(path, MAX_STR, infile) != NULL){
      strcpy(path+strlen(path)-1,"\0");
      sprintf(dest, "%s%s", srbParsed.dirname(),
          	path+strlen(fileParsed.dirname()));

//---
/*
          if(checkPoint){
                //dataObjOprInp.createMode = S_IFDIR;
                status = chkStateForResume (conn, &srbRestart, dest, COLL_OBJ_T, 1);
                if(status<0)
                        return status;
                else if(status==0)
                        continue;
          }
*/
//===

      status = srb_mkdir(conn, dest);
      if(status < 0){
        fprintf(stderr, "Collection creation error, status = %d\n", status);
        srb_perror (2, status, "", SRB_RCMD_ACTION|SRB_LONG_MSG);
        return -1;
      }
    }
    fclose(infile);
    // Finished Creating required directories at SRB

    // List all files in directory recursively and transfer
    sprintf(cmd, "find %s -type f", fileParsed.dirname());
    infile = popen(cmd,"r");
    if(infile == NULL){
      fprintf(stderr, "Error: Unable to open: tmp-SRBfilelist\n");
      return -1;
    }
    while(fgets(path, MAX_STR, infile) != NULL){
      strcpy(path+strlen(path)-1,"\0");
      sprintf(dest, "%s%s", srbParsed.dirname(),
          path+strlen(fileParsed.dirname()));
//---
          if(checkPoint){
                //dataObjOprInp.createMode = S_IFREG;
                status = chkStateForResume (conn, &srbRestart, dest, DATA_OBJ_T, 1);

                if(status<0)
                        return status;
                else if(status==0)
                        continue;
           }
//===

      // Call to transfer function
      status = transfer_from_file_to_srb(conn, path, dest,
          			filesizeVerify, checksumVerify, mymode, nthreads, resource);
      if(status < 0){
        fprintf(stderr, "Error: Transfer file %s to SRB %s failed!",
            path, dest);
        return -1;
      }
//---
	else {
                if (checkPoint){
                        srbRestart.curCnt ++;
                        status = writeRestartFile (&srbRestart, dest);
                }		
	}
//===
    }

    fclose(infile);
    // Finished Recursive Transfer files to SRB

  }
  /*********************** Wild Card Transfer ***************************/
  else if(fileParsed.basename() != NULL && srbParsed.basename() == NULL &&
      strchr(fileParsed.basename(), '*') != NULL){

    // Transfer files using wildcards

    FILE *infile;
    char path[MAX_STR];
    char dest[MAX_STR];
    char cmd[MAX_STR];

    // Create required directories at SRB
    status = srb_mkdir(conn, srbParsed.dirname());
    if(status < 0){
      fprintf(stderr, "Collection creation error, status = %d\n", status);
      srb_perror (2, status, "", SRB_RCMD_ACTION|SRB_LONG_MSG);
      return -1;
    }

    sprintf(cmd, "find %s -maxdepth 1 -iname \"%s\" -type f"
        ,fileParsed.dirname() , fileParsed.basename());
    infile = popen(cmd,"r");
    if(infile == NULL){
      fprintf(stderr, "Error: Unable to open: tmp-SRBfilelist\n");
      return -1;
    }

    while(fgets(path, MAX_STR, infile) != NULL){
      strcpy(path+strlen(path)-1,"\0");
      sprintf(dest, "%s%s", srbParsed.dirname(), strrchr(path, '/'));
//---
          if (checkPoint){
                //dataObjOprInp.createMode = S_IFREG;
                status = chkStateForResume (conn, &srbRestart, dest, DATA_OBJ_T, 1);
                if(status<0)
                        return status;
                else if(status==0)
                        continue;
           }
//===
      // Call to transfer function
      status = transfer_from_file_to_srb(conn, path, dest,
          filesizeVerify, checksumVerify, mymode, nthreads, resource);
      if(status < 0){
        fprintf(stderr, "Error: Transfer file %s to SRB %s failed!",
            path, dest);
        return -1;
      }
//---
      else{
            if (checkPoint){
                srbRestart.curCnt ++;
                status = writeRestartFile (&srbRestart, dest);
             }
      }
//===
    }

    fclose(infile);

  }
  /*********************** Single File Transfer ***************************/
  else if((fileParsed.basename() != NULL) &&
      (strchr(fileParsed.basename(), '*') == NULL)){
    // Perform a single file transfer from source to destination

    char dest[MAX_STR];
    int len = strlen(srbParsed.path());

    strncpy(dest, srbParsed.path(), len);


    // Create required directories at SRB
    status = srb_mkdir(conn, srbParsed.dirname());
    if(status < 0){
      fprintf(stderr, "Collection creation error, status = %d\n", status);
      srb_perror (2, status, "", SRB_RCMD_ACTION|SRB_LONG_MSG);
      return -1;
    }

    // Use source filename as destination filename if destination file name not
    // given
    if(srbParsed.basename() == NULL){
      len = strlen(srbParsed.path()) + strlen(fileParsed.basename());
      sprintf(dest, "%s/%s", srbParsed.dirname(), fileParsed.basename());
    }

    // Call to transfer function
    status = transfer_from_file_to_srb(conn, fileParsed.path(), dest,
          filesizeVerify, checksumVerify, mymode, nthreads, resource);
    if(status < 0){
      fprintf(stderr, "Error: Transfer file %s to SRB %s failed!",
          fileParsed.path(), dest);
      return -1;
    }
  }
  else{

    fprintf(stderr, "Please check the submitted urls and then try again\n");
    fprintf(stderr, "src:%s dest:%s\n", fileParsed.urlNoPassword(),
        srbParsed.urlNoPassword());

    return -1;
  }

  //------------------ Finished Transfering files to irods -------------------

//---
	//unlink("stork_file.rescue");
	unlink(file_ckpt_str);
//===

  clFinish(conn);

  return 0;

}
/*
int loop(srbConn *conn, Url& fileParsed, Url& srbParsed, char *collName,
    int filesizeVerify, int checksumVerify, int mymode, int nthreads, char *resource){
*/
int loop(srbConn *conn, Url& fileParsed, Url& srbParsed, char *collName,
    	 int filesizeVerify, int checksumVerify, int checkPoint, 
	  int mymode, int nthreads, char *resource){

  mdasC_sql_result_struct collResult, fileResult;
  char qval[MAX_DCS_NUM][MAX_TOKEN];
  int selval[MAX_DCS_NUM];
  int MAX_GET_RESULTS = 500;

  int i, j, status, fstatus = 1;
  //char *file, *source, *destination;
  char *file;
  //int mymode = 00755;
  //int mymode = getenv(mode);

//---
	int status1;
//===
  char source[MAX_STR];
  char destination[MAX_STR];

  memset(qval, '\0', MAX_DCS_NUM * MAX_TOKEN);
  memset(selval, 0, MAX_DCS_NUM);

  // Setting the collection name, filename and requesting DATA_NAME values
  // from the MCAT database

  sprintf(qval[PARENT_COLLECTION_NAME]," = '%s'",collName);
  selval[COLLECTION_NAME] = 1;

  // Retreiving the values from MCAT database
  if(((status = srbGetDataDirInfo(conn, catType, qval, selval,
            &collResult, MAX_GET_RESULTS)) < 0) && (status != -3005)){
    fprintf(stderr, "can't srbGetDataDirInfo. status = %d.\n", status);
    return -1;
  }

//---
//if there is no sub directories in the curr dir, but there are only files in it,
// and the status could be < 0, and it is one of the situations 
//===
  
  // start transfers
  //if(status == 0){
//---
  if (status ==0 || collResult.result_count == 0) {
//===
    memset(qval, '\0', MAX_DCS_NUM * MAX_TOKEN);
    memset(selval, 0, MAX_DCS_NUM);

    sprintf(qval[COLLECTION_NAME]," = '%s'",collName);
    selval[DATA_NAME] = 1;

    // Retreiving the values from MCAT database
    if(((fstatus = srbGetDataDirInfo(conn, catType, qval, selval,
              &fileResult, MAX_GET_RESULTS)) < 0) && (fstatus != -3005)){
      fprintf(stderr, "can't srbGetDataDirInfo. status = %d.\n", status);
      return -1;
    }

    do{
      for (i = 0; i < fileResult.result_count ; i++){
        file = (char *) fileResult.sqlresult[i].values;
        char *p = collName;
        p = p + strlen(srbParsed.dirname());
        /*
        if((source = (char *) malloc((strlen(collName) + 128) * sizeof (char))) == NULL){
          fprintf(stderr, "Error: memory allocation error for source\n");
          return -1;
        }

        if((destination = (char *) malloc((strlen(fileParsed.dirname()) + strlen(p) + 128) 
                * sizeof (char))) == NULL){
          fprintf(stderr, "Error: memory allocation error for destination\n");
          return -1;
        }
        */

        //sprintf(destination, "%s%s/%c", fileParsed.dirname(), p, '\0');
        // --- file_mkdir(destination);
        //printf("mkdir1:%s\n",destination);

        for (j = 0; j < fileResult.row_count; j++) {

          sprintf(source, "%s/%s%c", collName, file, '\0');
          sprintf(destination, "%s%s/%s%c", fileParsed.dirname(), p, file, '\0');
//---
          if(checkPoint){
                //dataObjOprInp.createMode = S_IFREG;
                status1 = chkStateForResume (conn, &srbRestart, destination, LOCAL_FILE_T, 1);

                if(status1<0)
                        return status;
                else if(status1==0){
			file += MAX_DATA_SIZE;
                        continue;
		}
           }	  
//===
          status = transfer_from_srb_to_file(conn, source, destination,
                    filesizeVerify, checksumVerify, mymode, nthreads, resource);
          if(status < 0){
            fprintf(stderr, "Error: transferring file from srb: %s to file: %s\n",
                source, destination);
            return -1;
          }
//---
          else {
                if (checkPoint){
                        srbRestart.curCnt ++;
                        status1 = writeRestartFile (&srbRestart, destination);
                }
          }	 
//===
          file += MAX_DATA_SIZE;
          printf("dst:%s\n",destination);
        }
      }

      if(fileResult.continuation_index >= 0){
        clearSqlResult(&fileResult);
        status = srbGetMoreRows(conn, catType, fileResult.continuation_index,
            &fileResult, MAX_GET_RESULTS);
        if(fileResult.result_count == 0 || status != 0){
          clFinish(conn);
          return -1;
        }
      }
    }while(fileResult.continuation_index >= 0);

  }

  do{
    for (i = 0; i < collResult.result_count ; i++){
      file = (char *) collResult.sqlresult[i].values;
      for (j = 0; j < collResult.row_count; j++) {

        char *p = file;
        p = p + strlen(srbParsed.dirname());
        sprintf(destination, "%s%s/%c", fileParsed.dirname(), p, '\0');
        //fprintf(stdout, "\nmkdir %s\n\n", destination);
        file_mkdir(destination, mymode);
/*
        loop(conn, fileParsed, srbParsed, file,
          filesizeVerify, checksumVerify, mymode, nthreads, resource);
*/
//---
        loop(conn, fileParsed, srbParsed, file,
          filesizeVerify, checksumVerify, checkPoint,
		mymode, nthreads, resource);	
//===
        file += MAX_DATA_SIZE;
      }
    }

    if(collResult.continuation_index >= 0){
      clearSqlResult(&collResult);
      status = srbGetMoreRows(conn, catType, collResult.continuation_index,
          &collResult, MAX_GET_RESULTS);
      if(collResult.result_count == 0 || status != 0){
        clFinish(conn);
        return -1;
      }
    }
  }while(collResult.continuation_index >= 0);

  return 0;

}



/* ============================================================================
 * Performing transfers from SRB to local filesystem
 * Parse submitted URLs
 * Obtain Stork Parameters
 * Connect to SRB server
 * Perform transfers according to type
 *  - Recursive
 *  - wildcard
 *  - single data object to file
 *  - else print error message
 * ==========================================================================*/

int srb_to_file(char *src, char *dst, int nthreads){

  srbConn *conn;
  int status;

  int checksumVerify = 0, filesizeVerify = 0, mymode = 0700;
  char resource[MAX_STR];
  resource[MAX_STR-1] = '\0';

  mdasC_sql_result_struct myresult;
  char qval[MAX_DCS_NUM][MAX_TOKEN];
  int selval[MAX_DCS_NUM];
  int MAX_GET_RESULTS = 500;

//---
	int checkPoint = 0;
   	srbPath_t targPath;
   	memset(&targPath, 0, sizeof(srbPath_t));
	int status1;
//===
  //------------------ Parsing URLs -----------------------------------------

  // Stork parse source (SRB) url
  Url srbParsed(src);
  if (! srbParsed.parsed()){
    fprintf(stderr, "parse src URL '%s' error\n",
        srbParsed.urlNoPassword() );
    return -1;
  }

  // Stork parse the source (SRB) url as SRB
  srbParsed.parseSrb();
  if (! srbParsed.parsedSrb()){
    fprintf(stderr, "parse src URL SRB fields'%s' error\n",
        srbParsed.urlNoPassword()
        );
    return -1;
  }

  // Stork parse destination (local) url
  Url fileParsed(dst);
  if (! fileParsed.parsed()){
    fprintf(stderr, "parse dest URL '%s' error\n",
        fileParsed.urlNoPassword() );
    return -1;
  }

  if(fileParsed.path() == NULL || srbParsed.path() == NULL ||
      srbParsed.host() == NULL || srbParsed.port() == NULL ||
      srbParsed.srbUser() == NULL || srbParsed.srbMdasDomain() == NULL ||
      srbParsed.password() == NULL){
    fprintf(stderr, "Error: Some fields are missing. Please recheck your URL "\
      "and resubmit the job.\nsrc: %s\ndest: %s\n", fileParsed.urlNoPassword(),
      srbParsed.urlNoPassword());
    return -1;
  }
  //------------------ Finished Parsing URLs ---------------------------------



  //------------------ Obtain Stork Parameters -------------------------------

  //set_stork_parameters(&filesizeVerify, &checksumVerify, &mymode, &nthreads, resource);
   set_stork_parameters(&filesizeVerify, &checksumVerify, &checkPoint, &mymode, &nthreads, resource);

  //------------------ Finish obtaining Stork Parameters ---------------------



  //------------------ Connecting to SRB -------------------------------------

  conn = connect_to_srb(srbParsed);

  //------------------ Finished Connecting to SRB ----------------------------



  //------------------ Transfering files to irods ----------------------------
 
//---
        convertCharStr(targPath.outPath, fileParsed.dirname());
        setStateForRestart (conn, &srbRestart, &targPath);
//===

  //********************** Recursive Transfer ***************************/

  if(srbParsed.dirname() != NULL && fileParsed.dirname() != NULL &&
      srbParsed.basename() == NULL && fileParsed.basename() == NULL){
    // Perform recursive copy of the directory given in the source url

/*
    status = loop(conn, fileParsed, srbParsed, srbParsed.dirname(), 
        filesizeVerify, checksumVerify, mymode, nthreads, resource);
*/
//---
    status = loop(conn, fileParsed, srbParsed, srbParsed.dirname(), 
        filesizeVerify, checksumVerify, checkPoint,
	mymode, nthreads, resource);	
//===
    if(status < 0){
      fprintf(stderr, "Error: recursive copy failed\n");
      clFinish(conn);
      return -1;
    }

  }
  /*********************** Wild Card Transfer ***************************/
  else if(srbParsed.basename() != NULL && fileParsed.basename() == NULL &&
      strchr(srbParsed.basename(), '*') != NULL){

    // Transfer files using wildcards

    char *filename, *file;

    if((filename = (char *) malloc(strlen(srbParsed.basename()) * sizeof(char))) == NULL){
      fprintf(stderr, "Error: memory allocation error for filename\n");
      return -1;
    }
    strcpy(filename, srbParsed.basename());

    // SQL database expects % and not * as wildcard
    while((file = strchr(filename, '*')) != NULL){
      *file = '%';
      file = strchr(filename, '*');
    }

    // Setting the values to 0s and empty strings
    for(status = 0; status < MAX_DCS_NUM; status++){
      selval[status] = 0;
      strcpy(qval[status], "");
    }

    // Setting the collection name, filename and requesting DATA_NAME values
    // from the MCAT database
    sprintf(qval[COLLECTION_NAME]," = '%s'",srbParsed.dirname());
    sprintf(qval[DATA_NAME]," like '%s'", filename);
    selval[DATA_NAME] = 1;

    // Retreiving the values from MCAT database
    if((status = srbGetDataDirInfo(conn, catType, qval, selval,
            &myresult, MAX_GET_RESULTS)) < 0){
      if(status == -3005){
        fprintf(stderr, "Error: Couldn't find any matching files %s\n", srbParsed.path());
      }
      fprintf(stderr, "can't srbGetDataDirInfo. status = %d.\n", status);
      clFinish(conn);
      free(filename);
      return -1;
    }

    //if (myresult == NULL) return -1;
    if(myresult.row_count == 0){
      fprintf(stderr, "query row_count returned is 0\n");
      clFinish(conn);
      free(filename);
      return -1;
    }

    char *source, *destination;
    if((source = (char *) malloc((strlen(srbParsed.dirname()) + 128) * sizeof (char))) == NULL){
      fprintf(stderr, "Error: memory allocation error for source\n");
      return -1;
    }

    if((destination = (char *) malloc((strlen(fileParsed.dirname()) + 128) * sizeof (char))) == NULL){
      fprintf(stderr, "Error: memory allocation error for destination\n");
      return -1;
    }

    status = file_mkdir(fileParsed.dirname(), mymode);
    if(status < 0){
      fprintf(stderr, "Error: creating directory structure status = %d\n", status);
      clFinish(conn);
      return -1;
    }


    // List files and perform transfer from SRB to file system
    // while checking for more files
    do{
      for (i = 0; i < myresult.result_count ; i++){
        file = (char *) myresult.sqlresult[i].values;
        for (j = 0; j < myresult.row_count; j++) {

          sprintf(source, "%s/%s%c", srbParsed.dirname(), file, '\0');
          sprintf(destination, "%s/%s%c", fileParsed.dirname(), file, '\0');
//---
          if(checkPoint){
                //dataObjOprInp.createMode = S_IFREG;
                status1 = chkStateForResume (conn, &srbRestart, destination, LOCAL_FILE_T, 1);
//---
		//printf("dest: %s\n", destination);
//===
                if(status1<0)
                        return status;
                else if(status1==0){
			file += MAX_DATA_SIZE;
                        continue;
		}
           }	  
//===
          status = transfer_from_srb_to_file(conn, source, destination,
                    filesizeVerify, checksumVerify, mymode, nthreads, resource);
          if(status < 0){
            fprintf(stderr, "Error: transfer from srb %s to file %s failed!\n", 
                source, destination);
            clFinish(conn);
            free(source);
            free(destination);
            free(filename);
            return -1;
          }
//---
          else {
                if (checkPoint){
                        srbRestart.curCnt ++;
                        status1 = writeRestartFile (&srbRestart, destination);
                }
          }	  
//===
          file += MAX_DATA_SIZE;
//---
	printf("dst:%s\n",destination);
//===

        }
      }

      if(myresult.continuation_index >= 0){
        clearSqlResult(&myresult);
        status = srbGetMoreRows(conn, catType, myresult.continuation_index,
            &myresult, MAX_GET_RESULTS);
        if(myresult.result_count == 0 || status != 0){
          clFinish(conn);
          free(source);
          free(destination);
          free(filename);
          return -1;
        }
      }
    }while(myresult.continuation_index >= 0);

    free(source);
    free(destination);
    free(filename);
  }
  /*********************** Single File Transfer ***************************/
  else if((srbParsed.basename() != NULL) &&
      (strchr(srbParsed.basename(), '*') == NULL)){
    // Perform a single file transfer from source to destination

    //char *dest = fileParsed.path();
    char *dest;
    int len = strlen(fileParsed.path());

    dest = (char *) malloc(len * sizeof(char));
    if(dest == NULL){
      fprintf(stderr, "Error: allocating memory\n");
      return -1;
    }

    strncpy(dest, fileParsed.path(), len);
    //strlcpy(dest, fileParsed.path(), len);


    // Create required directories at SRB
    status = file_mkdir(fileParsed.dirname(), mymode);
    if(status < 0){
      fprintf(stderr, "Error: creating directory structure status = %d\n", status);
      clFinish(conn);
      return -1;
    }

    // Use source filename as destination filename
    if(fileParsed.basename() == NULL){
      len = strlen(fileParsed.path()) + strlen(srbParsed.basename());
      //int len = strlen(fileParsed.path()) + strlen(srbParsed.basename());
      //if((dest = (char *) malloc(len * sizeof(char))) == NULL){
      dest = (char *) realloc(dest, len * sizeof(char));
      if(dest == NULL){
        fprintf(stderr, "Error: memory allocation error for dest\n");
        return -1;
      }

      sprintf(dest, "%s/%s%s", fileParsed.dirname(), srbParsed.basename(), "\0");
    }

    // Call to transfer function
    status = transfer_from_srb_to_file(conn, srbParsed.path(), dest,
                    filesizeVerify, checksumVerify, mymode, nthreads, resource);
    if(status < 0){
      fprintf(stderr, "Error: Transfer SRB %s to file %s failed!\n",
          srbParsed.path(), dest);
     clFinish(conn);
     free(dest);
     return -1;
    }
   free(dest);
  }
  else{
    fprintf(stderr, "Please check the submitted urls and then try again\n");
    fprintf(stderr, "src:%s dest:%s\n", srbParsed.urlNoPassword(),
        fileParsed.urlNoPassword());
    clFinish(conn);
    return -1;
  }

  //------------------ Finished Transfering files to irods -------------------

//---
  //unlink("stork_file.rescue");
	unlink(file_ckpt_str);
//===
  clFinish(conn);

  return 0;

}

//----------------------- MAIN BODY --------------------------------------

int main(int argc, char *argv[]){

  int status;
  int nthreads = -1;

  if (argc < 3){
    fprintf(stderr,
        "==============================================================\n");
    fprintf(stderr, "USAGE: %s <src_url> <dest_url>\n", argv[0]); fprintf(stderr,
        "==============================================================\n");
    return DAP_ERROR;
  }

  for(status = 0;status < argc;status++){
    if(strncmp(argv[status], "-p", 2) == 0){
      nthreads = atoi(argv[++status]);
      break;
    }
  }

  if (!strncmp(argv[1], "srb://",6) && !strncmp(argv[2] , "file:/", 6)){

    status = srb_to_file(argv[1], argv[2], nthreads);
    if(status < 0){
      fprintf(stderr, "Error: Unable to complete transfer to SRB.\n");
      return DAP_ERROR;
    }

  }else if (!strncmp(argv[1], "file:/", 6) && !strncmp(argv[2], "srb://", 6)){

    status = file_to_srb(argv[1], argv[2], nthreads);
    if(status < 0){
      fprintf(stderr, "Error: Unable to complete transfer from SRB.\n");
      return DAP_ERROR;
    }

  }else{
    fprintf(stderr, "Error: Transfer using this module not supported!\n");
    return DAP_ERROR;
  }

  return DAP_SUCCESS;

}
